from __future__ import unicode_literals

import base64
import os
import hashlib

import pdfkit
import six
from PIL import Image
from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage as storage
from django.template.loader import render_to_string


def create_screenshot_thumbnail(screenshot):
    pic = screenshot.image
    fh = storage.open(pic.name, 'r')
    image = Image.open(fh)
    image.thumbnail(settings.WALDUR_MARKETPLACE['THUMBNAIL_SIZE'], Image.ANTIALIAS)
    fh.close()

    thumb_extension = os.path.splitext(pic.name)[1]
    thumb_extension = thumb_extension.lower()
    thumb_name = os.path.basename(pic.name)

    if thumb_extension in ['.jpg', '.jpeg']:
        FTYPE = 'JPEG'
    elif thumb_extension == '.gif':
        FTYPE = 'GIF'
    elif thumb_extension == '.png':
        FTYPE = 'PNG'
    else:
        return

    temp_thumb = six.StringIO()
    image.save(temp_thumb, FTYPE)
    temp_thumb.seek(0)
    screenshot.thumbnail.save(thumb_name, ContentFile(temp_thumb.read()), save=True)
    temp_thumb.close()


def check_api_signature(data, api_secret_code, signature):
    return signature == get_api_signature(data, api_secret_code)


def get_api_signature(data, api_secret_code):
    concatenate_string = api_secret_code

    for usage in data['usages']:
        for key in sorted(usage.keys()):
            concatenate_string += key + six.text_type(usage[key])

    return hashlib.sha512(concatenate_string).hexdigest()


def create_order_pdf(order):
    logo_path = settings.WALDUR_CORE['SITE_LOGO']
    if logo_path:
        with open(logo_path, 'r') as image_file:
            deployment_logo = base64.b64encode(image_file.read())
    else:
        deployment_logo = None

    context = dict(
        order=order,
        currency=settings.WALDUR_CORE['CURRENCY_NAME'],
        deployment_name=settings.WALDUR_CORE['SITE_NAME'],
        deployment_address=settings.WALDUR_CORE['SITE_ADDRESS'],
        deployment_email=settings.WALDUR_CORE['SITE_EMAIL'],
        deployment_phone=settings.WALDUR_CORE['SITE_PHONE'],
        deployment_logo=deployment_logo,
    )
    html = render_to_string('marketplace/order.html', context)
    pdf = pdfkit.from_string(html, False)
    order.file = base64.b64encode(pdf)
    order.save()



from django.core.exceptions import ObjectDoesNotExist
from rest_framework.reverse import reverse
from rest_framework import serializers, status

from waldur_core.structure import models as structure_models
from waldur_mastermind.common.utils import internal_api_request
from waldur_openstack.openstack_tenant import apps as openstack_tenant_apps
from waldur_openstack.openstack_tenant import models as tenant_models
from waldur_openstack.openstack_tenant import views as tenant_views


def process_order_item(order_item, user):
    view = tenant_views.InstanceViewSet.as_view({'post': 'create'})
    post_data = get_post_data(order_item)
    response = internal_api_request(view, user, post_data)
    if response.status_code != status.HTTP_201_CREATED:
        raise serializers.ValidationError(response.data)

    order_item.scope = tenant_models.Instance.objects.get(uuid=response.data['uuid'])
    order_item.save()


def validate_order_item(order_item, request):
    post_data = get_post_data(order_item)
    serializer = tenant_views.InstanceViewSet.serializer_class(
        data=post_data, context={'request': request})
    serializer.is_valid(raise_exception=True)


def get_post_data(order_item):
    service_settings = order_item.offering.scope

    if not isinstance(service_settings, structure_models.ServiceSettings) or \
            service_settings.type != openstack_tenant_apps.OpenStackTenantConfig.service_name:
        raise serializers.ValidationError('Offering has invalid scope. Service settings object is expected.')

    project = order_item.order.project

    try:
        spl = tenant_models.OpenStackTenantServiceProjectLink.objects.get(
            project=project,
            service__settings=service_settings,
            service__customer=project.customer,
        )
    except ObjectDoesNotExist:
        raise serializers.ValidationError('Project does not have access to the OpenStack service.')

    payload = dict(
        service_project_link=reverse('openstacktenant-spl-detail', kwargs={'pk': spl.pk}),
    )

    fields = (
        'name',
        'description',
        'flavor',
        'image',
        'security_groups',
        'internal_ips_set',
        'floating_ips',
        'system_volume_size',
        'data_volume_size',
        'volumes',
        'ssh_public_key',
        'user_data',
    )
    for field in fields:
        if field in order_item.attributes:
            payload[field] = order_item.attributes.get(field)
    return payload



from __future__ import unicode_literals

from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils.functional import cached_property
from jsoneditor.forms import JSONEditor

from waldur_core.core import admin as core_admin
from waldur_core.structure import admin as structure_admin

from . import models


User = get_user_model()


class UserChoiceField(forms.ModelChoiceField):
    def label_from_instance(self, user):
        return '{} - {}'.format(user.full_name, user.username)


class SupportUserAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(SupportUserAdminForm, self).__init__(*args, **kwargs)
        self.fields['user'] = UserChoiceField(queryset=User.objects.all().order_by('full_name'))


class SupportUserAdmin(admin.ModelAdmin):
    form = SupportUserAdminForm


class OfferingAdminForm(forms.ModelForm):
    class Meta:
        widgets = {
            'report': JSONEditor(),
        }


class OfferingAdmin(admin.ModelAdmin):
    list_display = ('template', 'name', 'unit_price', 'unit', 'state')
    fields = ('name', 'unit_price', 'unit', 'template', 'issue',
              'project', 'state', 'product_code', 'article_code', 'report')
    form = OfferingAdminForm


class OfferingTemplateAdminForm(forms.ModelForm):
    class Meta:
        widgets = {
            'config': JSONEditor(),
        }


class OfferingTemplateAdmin(admin.ModelAdmin):
    form = OfferingTemplateAdminForm


class IssueAdmin(structure_admin.BackendModelAdmin):
    exclude = ('resource_content_type', 'resource_object_id')


class TemplateAttachmentInline(admin.TabularInline):
    model = models.TemplateAttachment
    fields = ('name', 'file')


class TemplateAdmin(core_admin.ExcludedFieldsAdminMixin,
                    admin.ModelAdmin):
    list_display = ('name', 'issue_type', 'created')
    search_fields = ('name', 'native_name')
    fields = ('name', 'native_name',
              'description', 'native_description',
              'issue_type', 'created', 'modified')
    readonly_fields = ('created', 'modified')
    inlines = [TemplateAttachmentInline]

    @cached_property
    def excluded_fields(self):
        if not settings.WALDUR_CORE['NATIVE_NAME_ENABLED']:
            return ['native_name', 'native_description']
        return []


class RequestTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'issue_type_name', 'backend_id')
    search_fields = ('name',)


admin.site.register(models.Offering, OfferingAdmin)
admin.site.register(models.Issue, IssueAdmin)
admin.site.register(models.Comment, structure_admin.BackendModelAdmin)
admin.site.register(models.Attachment)
admin.site.register(models.SupportUser, SupportUserAdmin)
admin.site.register(models.Template, TemplateAdmin)
admin.site.register(models.OfferingTemplate, OfferingTemplateAdmin)
admin.site.register(models.TemplateStatusNotification)
admin.site.register(models.IgnoredIssueStatus)
admin.site.register(models.RequestType, RequestTypeAdmin)



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction

from waldur_core.structure import models as structure_models
from waldur_mastermind.marketplace import models as marketplace_models
from waldur_openstack.openstack_tenant import apps as openstack_tenant_apps

from . import PLUGIN_NAME

logger = logging.getLogger(__name__)


def create_offering_from_tenant(sender, instance, created=False, **kwargs):
    if created:
        return

    if not instance.tracker.has_changed('state'):
        return

    if instance.tracker.previous('state') != instance.States.CREATING:
        return

    if instance.state != instance.States.OK:
        return

    tenant = instance

    try:
        order_item = marketplace_models.OrderItem.objects.get(scope=tenant)
    except ObjectDoesNotExist:
        logger.debug('Skipping offering creation for tenant because order '
                     'item does not exist. OpenStack tenant ID: %s', tenant.id)
        return

    try:
        service_settings = structure_models.ServiceSettings.objects.get(
            scope=tenant,
            type=openstack_tenant_apps.OpenStackTenantConfig.service_name,
        )
    except ObjectDoesNotExist:
        logger.debug('Skipping offering creation for tenant because service settings '
                     'object does not exist. OpenStack tenant ID: %s', tenant.id)
        return

    parent_offering = order_item.offering
    payload = dict(
        type=PLUGIN_NAME,
        name='Virtual machine in %s' % parent_offering.name,
        scope=service_settings,
        shared=False,
    )

    fields = (
        'state',
        'customer',
        'category',
        'attributes',
        'thumbnail',
        'vendor_details',
        'geolocations',
    )
    for field in fields:
        payload[field] = getattr(parent_offering, field)

    with transaction.atomic():
        offering = marketplace_models.Offering.objects.create(**payload)
        offering.allowed_customers.add(tenant.service_project_link.project.customer)


def archive_offering(sender, instance, **kwargs):
    tenant = instance

    try:
        service_settings = structure_models.ServiceSettings.objects.get(
            scope=tenant,
            type=openstack_tenant_apps.OpenStackTenantConfig.service_name,
        )
    except ObjectDoesNotExist:
        logger.debug('Skipping offering creation for tenant because service settings '
                     'object does not exist. OpenStack tenant ID: %s', tenant.id)
        return

    marketplace_models.Offering.objects.filter(scope=service_settings).update(
        state=marketplace_models.Offering.States.ARCHIVED
    )


def change_order_item_state(sender, instance, created=False, **kwargs):
    if created:
        return

    if not instance.tracker.has_changed('state'):
        return

    if instance.tracker.previous('state') != instance.States.CREATING:
        return

    if instance.state in [instance.States.OK, instance.States.ERRED]:
        try:
            order_item = marketplace_models.OrderItem.objects.get(scope=instance)
        except ObjectDoesNotExist:
            logger.debug('Skipping OpenStack instance state synchronization with marketplace '
                         'because order item does not exist. OpenStack instance ID: %s', instance.id)
            return

        if instance.state == instance.States.OK:
            order_item.set_state_done()
            order_item.save(update_fields=['state'])

        if instance.state == instance.States.ERRED:
            order_item.set_state_erred()
            order_item.save(update_fields=['state'])
